@@ -2,5 +2,6 @@
 
 from .client import Client
 from .server import Server, respond
+from .core.token import token_create
 
-__all__ = ["Client", "Server", "respond"]
+__all__ = ["Client", "Server", "respond", "token_create"]
