@@ -1,8 +1,8 @@
-from ..sockets import ClientSocket
-from ..interface import respond
+from ..core.sockets import ClientSocket
+from ..server import respond
 
 
-def init_task(client_socket: ClientSocket) -> dict:
+def init_task() -> dict:
     init_args = {"request_count": 0, "value": 1000}
     return init_args
 
